<?php

namespace Bydedi\Catalog\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Knp\DoctrineBehaviors\Model as ORMBehaviors;

/**
 * Class Product
 * @package Bydedi\Catalog\Entity
 *
 * @ORM\MappedSuperclass
 */
abstract class Product
{
    use ORMBehaviors\Translatable\Translatable;
    use ORMBehaviors\Timestampable\Timestampable;
    use ORMBehaviors\SoftDeletable\SoftDeletable;

    /**
     * @ORM\Id
     * @ORM\Column(type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @var integer
     *
     * @ORM\Column(name="delivery_time", type="integer", nullable=true)
     */
    protected $deliveryTime;

    /**
     * @var integer
     *
     * @ORM\Column(name="quantity_info", type="integer", nullable=false)
     */
    protected $quantity = 0;

    /**
     * @var integer
     *
     * @ORM\Column(name="quantity_phy", type="integer", nullable=false)
     */
    protected $quantityPhy;

    /**
     * @var integer
     *
     * @ORM\Column(name="alerte", type="integer", nullable=true)
     */
    protected $alerte = null;

    /**
     * @var integer
     *
     * @ORM\Column(name="unit", type="integer", nullable=false)
     */
    protected $unit = 1;

    /**
     * @var string
     *
     * @ORM\Column(name="reference", type="string", length=255, nullable=false)
     */
    protected $reference = '';

    /**
     * @var string
     *
     * @ORM\Column(name="reference_supplier", type="string", length=255)
     */
    protected $referenceSupplier = '';

    /**
     * @var string
     *
     * @ORM\Column(name="price", type="decimal", precision=15, scale=4, nullable=false)
     */
    protected $price = 0.0000;

    /**
     * @var string
     *
     * @ORM\Column(name="ecotaxe", type="decimal", precision=15, scale=4, nullable=false)
     */
    protected $ecotaxe;

    /**
     * @var string
     *
     * @ORM\Column(name="price_advisable", type="decimal", precision=15, scale=4, nullable=false)
     */
    protected $priceAdvisable;

    /**
     * @var string
     *
     * @ORM\Column(name="price_purchase", type="decimal", precision=15, scale=4, nullable=false)
     */
    protected $pricePurchase = 0.0000;

    /**
     * @var string
     *
     * @ORM\Column(name="price_purchase_last", type="decimal", precision=15, scale=4, nullable=false)
     */
    protected $pricePurchaseLast;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer", nullable=false)
     */
    protected $weight = 0;

    /**
     * @var integer
     *
     * @ORM\Column(name="produit_statut_id", type="integer")
     */
    protected $statut;

    /**
     * @var integer
     *
     * @ORM\Column(name="rank", type="integer", nullable=false)
     */
    protected $rank;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer", nullable=false)
     */
    protected $appro = 0;

    /**
     * @var string
     *
     * @ORM\Column(name="ean", type="string", length=255, nullable=false)
     */
    protected $ean;

    /**
     * @var integer
     *
     * @ORM\Column(name="quantity_limit", type="integer", nullable=true)
     */
    protected $quantityLimit;

    /**
     * @var \Bydedi\Catalog\Entity\Tax
     *
     * @ORM\ManyToOne(targetEntity="Bydedi\Catalog\Entity\Tax")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="tax_id", referencedColumnName="id")
     * })
     */
    protected $tax;

    /**
     * @var \Bydedi\Catalog\Entity\Brand
     *
     * @ORM\ManyToOne(targetEntity="Bydedi\Catalog\Entity\Brand")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="brand_id", referencedColumnName="id")
     * })
     */
    protected $brand;

    /**
     * @var \Bydedi\Catalog\Entity\ProductType
     *
     * @ORM\ManyToOne(targetEntity="Bydedi\Catalog\Entity\ProductType")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="product_type_id", referencedColumnName="id")
     * })
     */
    protected $type;

    /**
     * @var \Entity\Shop[]
     *
     * @ORM\ManyToMany(targetEntity="Entity\Shop")
     * @ORM\JoinTable(name="product_shop",
     *      joinColumns={@ORM\JoinColumn(name="product_id", referencedColumnName="id")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="shop_id", referencedColumnName="id", unique=true)}
     *      )
     */
    protected $shop;

    /**
     * @ORM\ManyToMany(targetEntity="Bydedi\Catalog\Entity\FeatureValue", inversedBy="products")
     * @ORM\JoinTable(name="product_feature_value",
     *      joinColumns={@ORM\JoinColumn(name="product_id", referencedColumnName="id")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="feature_value_id", referencedColumnName="id")}
     *     )
     */
    protected $featureValues;

    /**
     * @ORM\ManyToMany(targetEntity="Bydedi\Catalog\Entity\Category", inversedBy="products")
     * @ORM\JoinTable(name="product_category")
     */
    protected $categories;

    /**
     * Product constructor.
     */
    public function __construct()
    {
        $this->featureValues = new ArrayCollection();
        $this->categories = new ArrayCollection();
    }

    /**
     * @param $method
     * @param $arguments
     * @return mixed
     */
    public function __call($method, $arguments)
    {
        return $this->proxyCurrentLocaleTranslation($method, $arguments);
    }

    /**
     * @return mixed
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param mixed $id
     * @return Product
     */
    public function setId($id)
    {
        $this->id = $id;
        return $this;
    }

    /**
     * @return int
     */
    public function getDeliveryTime()
    {
        return $this->deliveryTime;
    }

    /**
     * @param int $deliveryTime
     * @return Product
     */
    public function setDeliveryTime($deliveryTime)
    {
        $this->deliveryTime = $deliveryTime;
        return $this;
    }

    /**
     * @return int
     */
    public function getQuantity()
    {
        return $this->quantity;
    }

    /**
     * @param int $quantity
     * @return Product
     */
    public function setQuantity($quantity)
    {
        $this->quantity = $quantity;
        return $this;
    }

    /**
     * @return int
     */
    public function getQuantityPhy()
    {
        return $this->quantityPhy;
    }

    /**
     * @param int $quantityPhy
     * @return Product
     */
    public function setQuantityPhy($quantityPhy)
    {
        $this->quantityPhy = $quantityPhy;
        return $this;
    }

    /**
     * @return int
     */
    public function getAlerte()
    {
        return $this->alerte;
    }

    /**
     * @param int $alerte
     * @return Product
     */
    public function setAlerte($alerte)
    {
        $this->alerte = $alerte;
        return $this;
    }

    /**
     * @return int
     */
    public function getUnit()
    {
        return $this->unit;
    }

    /**
     * @param int $unit
     * @return Product
     */
    public function setUnit($unit)
    {
        $this->unit = $unit;
        return $this;
    }

    /**
     * @return string
     */
    public function getReference()
    {
        return $this->reference;
    }

    /**
     * @param string $reference
     * @return Product
     */
    public function setReference($reference)
    {
        $this->reference = $reference;
        return $this;
    }

    /**
     * @return string
     */
    public function getReferenceSupplier()
    {
        return $this->referenceSupplier;
    }

    /**
     * @param string $referenceSupplier
     * @return Product
     */
    public function setReferenceSupplier($referenceSupplier)
    {
        $this->referenceSupplier = $referenceSupplier;
        return $this;
    }

    /**
     * @return string
     */
    public function getPrice()
    {
        return $this->price;
    }

    /**
     * @param string $price
     * @return Product
     */
    public function setPrice($price)
    {
        $this->price = $price;
        return $this;
    }

    /**
     * @return string
     */
    public function getEcotaxe()
    {
        return $this->ecotaxe;
    }

    /**
     * @param string $ecotaxe
     * @return Product
     */
    public function setEcotaxe($ecotaxe)
    {
        $this->ecotaxe = $ecotaxe;
        return $this;
    }

    /**
     * @return string
     */
    public function getPriceAdvisable()
    {
        return $this->priceAdvisable;
    }

    /**
     * @param string $priceAdvisable
     * @return Product
     */
    public function setPriceAdvisable($priceAdvisable)
    {
        $this->priceAdvisable = $priceAdvisable;
        return $this;
    }

    /**
     * @return string
     */
    public function getPricePurchase()
    {
        return $this->pricePurchase;
    }

    /**
     * @param string $pricePurchase
     * @return Product
     */
    public function setPricePurchase($pricePurchase)
    {
        $this->pricePurchase = $pricePurchase;
        return $this;
    }

    /**
     * @return string
     */
    public function getPricePurchaseLast()
    {
        return $this->pricePurchaseLast;
    }

    /**
     * @param string $pricePurchaseLast
     * @return Product
     */
    public function setPricePurchaseLast($pricePurchaseLast)
    {
        $this->pricePurchaseLast = $pricePurchaseLast;
        return $this;
    }

    /**
     * @return int
     */
    public function getWeight()
    {
        return $this->weight;
    }

    /**
     * @param int $weight
     * @return Product
     */
    public function setWeight($weight)
    {
        $this->weight = $weight;
        return $this;
    }

    /**
     * @return int
     */
    public function getStatut()
    {
        return $this->statut;
    }

    /**
     * @param int $statut
     * @return Product
     */
    public function setStatut($statut)
    {
        $this->statut = $statut;
        return $this;
    }

    /**
     * @return int
     */
    public function getRank()
    {
        return $this->rank;
    }

    /**
     * @param int $rank
     * @return Product
     */
    public function setRank($rank)
    {
        $this->rank = $rank;
        return $this;
    }

    /**
     * @return int
     */
    public function getAppro()
    {
        return $this->appro;
    }

    /**
     * @param int $appro
     * @return Product
     */
    public function setAppro($appro)
    {
        $this->appro = $appro;
        return $this;
    }

    /**
     * @return string
     */
    public function getEan()
    {
        return $this->ean;
    }

    /**
     * @param string $ean
     * @return Product
     */
    public function setEan($ean)
    {
        $this->ean = $ean;
        return $this;
    }

    /**
     * @return int
     */
    public function getQuantityLimit()
    {
        return $this->quantityLimit;
    }

    /**
     * @param int $quantityLimit
     * @return Product
     */
    public function setQuantityLimit($quantityLimit)
    {
        $this->quantityLimit = $quantityLimit;
        return $this;
    }

    /**
     * @return Tax
     */
    public function getTax()
    {
        return $this->tax;
    }

    /**
     * @param Tax $tax
     * @return Product
     */
    public function setTax($tax)
    {
        $this->tax = $tax;
        return $this;
    }

    /**
     * @return Brand
     */
    public function getBrand()
    {
        return $this->brand;
    }

    /**
     * @param Brand $brand
     * @return Product
     */
    public function setBrand($brand)
    {
        $this->brand = $brand;
        return $this;
    }

    /**
     * @return ProductType
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param ProductType $type
     * @return Product
     */
    public function setType($type)
    {
        $this->type = $type;
        return $this;
    }

    /**
     * @return FeatureValue[]
     */
    public function getFeatureValues()
    {
        return $this->featureValues;
    }
}
