<?php

namespace Bydedi\Catalog;

use Bydedi\Catalog\Model\Brand;
use Bydedi\Catalog\Model\Category;
use Bydedi\Catalog\Model\Feature;
use Bydedi\Catalog\Model\FeatureValue;
use Bydedi\Catalog\Model\Product;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class Listing
 */
class Listing
{

    /**
     * @var Product[]
     */
    protected $products;
    /**
     * @var FeatureValue[]
     */
    protected $features;
    /**
     * @var array
     */
    protected $caraProducts;
    /**
     * @var Brand[]
     */
    protected $brands;
    /**
     * @var Category[]
     */
    protected $categories;
    /**
     * @var string
     */
    protected $search;
    /**
     * @var array
     */
    protected $prices;
    /**
     * @var int
     */
    protected $page;
    /**
     * @var int
     */
    protected $limit;
    /**
     * @var string
     */
    protected $group;
    /**
     * @var string
     */
    protected $order;
    /**
     * @var string
     */
    protected $featureUrl;
    /**
     * @var array
     */
    protected $statut;

    /**
     * @var \Symfony\Component\HttpFoundation\RequestStack
     */
    protected $requestStack;
    /**
     * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
     */
    protected $dispatcher;

    /**
     * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $dispatcher
     * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
     */
    public function __construct(EventDispatcherInterface $dispatcher, RequestStack $requestStack)
    {
        $this->dispatcher = $dispatcher;
        $this->requestStack = $requestStack;
        $this->reset();
    }

    /**
     * init the object
     */
    public function reset()
    {

        $this->page = 1;
        $this->limit = null;
        $this->order = null;
        $this->group = '';
        $this->brands = array();
        $this->resetCategories();
        $this->resetProducts();
        $this->caraProducts = array();
        $this->features = array();
        $this->search = '';
        $this->prices = array();
        $this->statut = array();

        return $this;
    }

    /**
     * @return $this
     */
    public function resetCategories()
    {
        $this->categories = array();
        return $this;
    }

    /**
     * @return $this
     */
    public function resetProducts()
    {
        $this->products = array();
        return $this;
    }

    /**
     * @param int $statut
     * * @return $this
     */
    public function addStatut($statut)
    {
        $this->statut[] = $statut;
        return $this;
    }

    /**
     * @param Category $category
     * @return $this
     */
    public function addCategory(Category $category)
    {
        $this->categories[] = $category;

        return $this;
    }

    /**
     * @param Brand $brand
     * @return $this
     */
    public function addBrand(Brand $brand)
    {
        $this->brands[] = $brand;
        return $this;
    }

    /**
     * @param FeatureValue $featureValue
     * @return $this
     */
    public function addFeature(FeatureValue $featureValue)
    {
        $this->features[] = $featureValue;
        return $this;
    }

    /**
     * @param $search string
     * * @return $this
     */
    public function setSearch($search)
    {
        $this->search = $search;
        return $this;
    }

    /**
     * @param $page int
     * * @return $this
     */
    public function setPage($page)
    {
        $this->page = $page;
        return $this;
    }


    /**
     * @param $order string
     * @return $this
     */
    public function setOrder($order)
    {
        $this->order = $order;
        return $this;
    }

    /**
     * @param $limit int
     * @return $this
     */
    public function setLimit($limit)
    {
        $this->limit = $limit;
        return $this;
    }

    /**
     * @param $group string
     * @return $this
     */
    public function setGroup($group)
    {
        $this->group = $group;
        return $this;
    }


    /**
     * @param $prices
     * @return $this
     */
    public function setPrices($prices)
    {
        $this->prices = $prices;
        return $this;
    }

    /**
     * function to get all products match with the criteria add : Categorie / Gamme / Marque / etc...
     *
     * @return array
     */

    public function getProducts()
    {
        $this->loadProductsIfNotLoaded();
        return $this->products;
    }

    /**
     *
     * @return array retourne un tableau avec les marques des produits de la selection
     */
    public function getBrands()
    {
        $this->loadProductsIfNotLoaded();

        // @TODO
    }

    /**
     *
     * @return array retourne un tableau avec les marques des produits de la selection
     */
    public function getCategories()
    {
        $this->loadProductsIfNotLoaded();

        // @TODO
    }

    /**
     * function to get just the product to display on the page, use page and limit
     *
     * @param array $param
     *
     * @return array
     *
     */
    public function getProductsToDisplay($param = array())
    {
        $this->loadProductsIfNotLoaded($param);

        $produit = new Product();

        $offset = $this->limit * ($this->page - 1);
        $productsSlice = array_slice($this->products, $offset, $this->limit);

        $productsToDisplay = [];

        foreach ($productsSlice as $ps) {

            $p = $produit->getOne([
                'produit_id' => $ps['produit_id'],
                'produit_attribut_id' => $ps['produit_attribut_id'],
            ]);

            if (!empty($p)) {
                $productsToDisplay[] = $p;
            }
        }

        return $productsToDisplay;
    }

    /**
     * @return int
     */
    public function getCountProducts()
    {
        return count($this->products);
    }

    /**
     * TODO : a revoir
     *
     * @return array
     */
    public function getPagination()
    {
        $pagination = array();

        $total = $this->getCountProducts();
        $first = ($this->page - 1) * $this->limit + 1;
        if ($first > $total) {
            $first = $total;
        }
        $last = ($this->page) * $this->limit;
        if ($last > $total) {
            $last = $total;
        }
        $pages = ceil($total / $this->limit);

        $pas = 10;
        $tronc = floor($this->page / $pas);

        if ($tronc == 0) {
            $pagination['INTMIN'] = 1;
            $pagination['INTMAX'] = 9;
            $pagination['PAS'] = $pas - 1;
        } else {
            $pagination['INTMIN'] = $tronc * $pas;
            $pagination['INTMAX'] = ($tronc * $pas) + $pas;
            $pagination['PAS'] = $pas;
        }

        // Affect des donnees de pagination
        $pagination['NBROW'] = $total;
        $pagination['FIRSTROW'] = $first;
        $pagination['LASTROW'] = $last;
        $pagination['NBPAGE'] = $pages;
        $pagination['CURRENTPAGE'] = ($this->page > $pages) ? $pages : $this->page;

        if ($total > 0) {
            $pagination['start'] = ($pagination['CURRENTPAGE'] - 3 < 1) ? 1 : $pagination['CURRENTPAGE'] - 3;
            $pagination['end'] = ($pagination['start'] + 5 > $pagination['NBPAGE']) ?
                $pagination['NBPAGE'] : $pagination['start'] + 5;
        } else {
            $pagination['start'] = 1;
            $pagination['end'] = 1;
        }

        $pagination['prec'] = $pagination['CURRENTPAGE'] - 1;
        $pagination['suiv'] = $pagination['CURRENTPAGE'] + 1;

        return $pagination;
    }

    /**
     * @return array
     */
    public function getFeatureSelect()
    {
        $arrCara = array();
        if (!empty($this->features)) {
            $feature = new Feature();

            $request = $this->requestStack->getCurrentRequest();

            $url = (!empty($this->featureUrl)) ?
                str_replace('/' . $this->featureUrl, '', $request->getRequestUri()) :
                $request->getRequestUri();

            foreach ($this->features as $fea) {
                $caraTmp = array($fea->toArray());
                $feature->setUrl($caraTmp, $this->features, $url);
                $arrCara[] = $caraTmp[0];
            }
        }

        return $arrCara;
    }

    /**
     * function to get all cara match with the product in the listing
     *
     * @param null|int $featureId
     * @return array
     */
    public function getCaracteristique($featureId = null)
    {
        if (empty($this->caraProducts)) {
            $this->loadFeature();
        }

        if (!is_null($featureId)) {
            if (!is_array($featureId)) {
                $featureId = array($featureId);
            }

            $caracs = array_map(
                function ($value) use ($featureId) {
                    if (in_array($value['caracteristique_id'], $featureId)) {
                        return $value;
                    }

                    return null;
                },
                $this->caraProducts
            );

            return array_filter($caracs);
        }

        return $this->caraProducts;
    }

    /**
     * @param $url
     */
    public function setFeatureFromUrl($url)
    {
        $this->featureUrl = $url;

        $caracteristique = new Feature();

        $features = $caracteristique->parseUrl($this->featureUrl);

        foreach ($features as $id) {
            $this->addFeature(new FeatureValue($id));
        }
    }

    protected function loadProductsIfNotLoaded($param = array())
    {
        if (empty($this->products)) {
            $this->loadProducts($param);
        }
    }

    /**
     * function to load all products match with the criteria add : Categorie / Gamme / Marque / etc...
     *
     * @param array $param
     *
     */
    protected function loadProducts($param = array())
    {
        $produit = new Product();

        $param['noBloc'] = true;

        if (!empty($this->marques)) {
            foreach ($this->marques as $m) {
                $param['marque'][] = $m->marque_id;
            }
        }

        if (!empty($this->categories)) {
            $param['categorie_ext_id'] = $this->categories[0]->categorie_id;
        }

        if (!empty($this->gammes)) {
            foreach ($this->gammes as $g) {
                if (!empty($g->gamme_id)) {
                    $param['gamme'][] = $g->gamme_id;
                }
            }
        }

        if (!empty($this->caracteristiques)) {
            foreach ($this->caracteristiques as $c) {
                $param['caracteristique'][] = $c->caracteristique_valeur_id;
            }
        }

        if (!empty($this->prix)) {
            $param['prix'] = $this->prix;
        }

        if (!empty($this->search)) {
            if (is_array($this->search)) {
                list($idx, $search) = $this->search;
            } else {
                $idx = 'motclef';
                $search = $this->search;
            }
            $param[$idx] = $search;
        }

        if (!empty($this->statut)) {
            $param['statut'] = $this->statut;
        }

        $param['order'] = $this->order;
        $param['group'] = $this->group;

        $this->products = $produit->getSearch($param);
    }

    /**
     *
     * function to load the list of the cara for the products
     */
    protected function loadFeature()
    {
        $caracteristique = new Caracteristique();

        $arrValeur = $caracteristique->getValeurProduit([
            'listProduit' => Tools::extractValue('produit_id', $this->products)
        ]);

        if (!empty($arrValeur)) {
            $url = (!empty($this->caracteristiqueUrl)) ?
                str_replace('/' . $this->caracteristiqueUrl, '', $this->request->getRequestUri()) :
                $this->request->getRequestUri();

            $caracteristique->setUrl($arrValeur, $this->caracteristiques, $url);
        }

        $this->caraProducts = $caracteristique->constructCara($arrValeur, $this->caracteristiques, $this->products);
    }

    /**
     * @return array
     */
    public function getReferencement()
    {
        // TODO : a revoir
        /*
        $texte = new Texte();

        $ref = array(
            'title'       => '',
            'description' => '',
            'keywords'    => '',
            'other'       => ''
        );

        if (!empty($this->categories)) {
            $type = 'categorie';
            $suffixeTxt = (empty($this->categories[0]->parent_id)) ? 'parent' : 'categorie';
            $obj = $this->categories[0];
            if (!empty($this->categories[0]->parent_id)) {
                $parent = new Categorie($this->categories[0]->parent_id);
                $obj->parent_nom_nom = $parent->categorie_nom_nom;
            }
        } elseif (!empty($this->marques)) {
            $type = $suffixeTxt = 'marque';
            $obj = $this->marques[0];
        } elseif (!empty($this->gammes)) {
            $type = $suffixeTxt = 'gamme';
            $obj = $this->gammes[0];
        } elseif (!empty($this->search)) {
            $ref['title'] = $texte->getOne('ref_title_search', true, array('motclef' => $this->search));
            $ref['description'] = $texte->getOne('ref_desc_search', true, array('motclef' => $this->search));
        }

        if (!empty($suffixeTxt) && !empty($obj)) {
            $ref['title'] = (!empty($obj->{$type . '_nom_meta_title'})) ? $obj->{$type . '_nom_meta_title'} : $texte->getOne('ref_title_' . $suffixeTxt,
                true, $obj->toArray());
            $ref['description'] = (!empty($obj->{$type . '_nom_meta_desc'})) ? $obj->{$type . '_nom_meta_desc'} : $texte->getOne('ref_desc_' . $suffixeTxt,
                true, $obj->toArray());
            $ref['keywords'] = (!empty($obj->{$type . '_nom_meta_kw'})) ? $obj->{$type . '_nom_meta_kw'} : $texte->getOne('ref_kw_' . $suffixeTxt,
                true, $obj->toArray());
            $ref['other'] = (!empty($obj->{$type . '_nom_meta_other'})) ? $obj->{$type . '_nom_meta_other'} : $texte->getOne('ref_other_' . $suffixeTxt,
                true, $obj->toArray());
        }

        return $ref;
        */
    }
}
